Index: channels/chan_sip.c
===================================================================
--- channels/chan_sip.c	(revision 430794)
+++ channels/chan_sip.c	(working copy)
@@ -31075,38 +31075,31 @@
 	}
 }
 
-static void cleanup_all_regs(void)
+static int cleanup_all_regs(void *reg_obj, void *arg, int flags)
 {
-	struct ao2_iterator iter;
-	struct sip_registry *iterator;
-	/* First, destroy all outstanding registry calls */
-	/* This is needed, since otherwise active registry entries will not be destroyed */
-	iter = ao2_iterator_init(registry_list, 0);
-	while ((iterator = ao2_t_iterator_next(&iter, "cleanup_all_regs iter"))) {
-		ao2_lock(iterator);
+	struct sip_registry *iterator = reg_obj;
+	ao2_lock(iterator);
 
-		if (iterator->call) {
-			ast_debug(3, "Destroying active SIP dialog for registry %s@%s\n", iterator->username, iterator->hostname);
-			/* This will also remove references to the registry */
-			dialog_unlink_all(iterator->call);
-			iterator->call = dialog_unref(iterator->call, "remove iterator->call from registry traversal");
-		}
-		if (iterator->expire > -1) {
-			AST_SCHED_DEL_UNREF(sched, iterator->expire, ao2_t_ref(iterator, -1, "reg ptr unref from reload config"));
-		}
-		if (iterator->timeout > -1) {
-			AST_SCHED_DEL_UNREF(sched, iterator->timeout, ao2_t_ref(iterator, -1, "reg ptr unref from reload config"));
-		}
-		if (iterator->dnsmgr) {
-			ast_dnsmgr_release(iterator->dnsmgr);
-			iterator->dnsmgr = NULL;
-			ao2_t_ref(iterator, -1, "reg ptr unref from dnsmgr");
-		}
+	if (iterator->call) {
+		ast_debug(3, "Destroying active SIP dialog for registry %s@%s\n", iterator->username, iterator->hostname);
+		/* This will also remove references to the registry */
+		dialog_unlink_all(iterator->call);
+		iterator->call = dialog_unref(iterator->call, "remove iterator->call from registry traversal");
+	}
+	if (iterator->expire > -1) {
+		AST_SCHED_DEL_UNREF(sched, iterator->expire, ao2_t_ref(iterator, -1, "reg ptr unref from reload config"));
+	}
+	if (iterator->timeout > -1) {
+		AST_SCHED_DEL_UNREF(sched, iterator->timeout, ao2_t_ref(iterator, -1, "reg ptr unref from reload config"));
+	}
+	if (iterator->dnsmgr) {
+		ast_dnsmgr_release(iterator->dnsmgr);
+		iterator->dnsmgr = NULL;
+		ao2_t_ref(iterator, -1, "reg ptr unref from dnsmgr");
+	}
 
-		ao2_unlock(iterator);
-		ao2_t_ref(iterator, -1, "cleanup_all_regs iter");
-	}
-	ao2_iterator_destroy(&iter);
+	ao2_unlock(iterator);
+	return CMP_MATCH;
 }
 
 /*! \brief Re-read SIP.conf config file
@@ -31191,7 +31184,8 @@
 		ast_mutex_unlock(&authl_lock);
 
 		/* Then, actually destroy users and registry */
-		cleanup_all_regs();
+		ao2_callback(registry_list, OBJ_NODATA | OBJ_UNLINK | OBJ_MULTIPLE,
+			cleanup_all_regs, "remove all SIP registry items");
 		ast_debug(4, "--------------- Done destroying registry list\n");
 		ao2_t_callback(peers, OBJ_NODATA, peer_markall_func, NULL, "callback to mark all peers");
 	}
@@ -34579,7 +34573,8 @@
 	ast_free(default_tls_cfg.cafile);
 	ast_free(default_tls_cfg.capath);
 
-	cleanup_all_regs();
+	ao2_callback(registry_list, OBJ_NODATA | OBJ_UNLINK | OBJ_MULTIPLE,
+		cleanup_all_regs, "remove all SIP registry items");
 	ao2_cleanup(registry_list);
 
 	{
